/* $Id: isaac_img_lingain.c,v 1.3 2009/04/28 14:45:17 llundin Exp $
 *
 * This file is part of the DETMON Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2009/04/28 14:45:17 $
 * $Revision: 1.3 $
 * $Name: HEAD $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*----------------------------------------------------------------------------
                             Includes and Defines
 ----------------------------------------------------------------------------*/

#include <cpl.h>

#include "irplib_detmon.h"
#include "irplib_detmon_lg.h"
#include "irplib_plugin.h"

/* Define here the DO.CATG keywords */
#define ISAAC_IMG_LINGAIN_ON_RAW                    "CAL_LINGAIN_LAMP"
#define ISAAC_IMG_LINGAIN_OFF_RAW		    "CAL_LINGAIN_DARK"

#define RECIPE_NAME "isaac_img_lingain"

/* Copy here instrument specific keywords which need to be in the PAF file */
#define INSTREGEXP   "ESO DET CHIP NAME|ESO DET MODE NAME"

#define PAFREGEXP    "^(" REGEXP "|" INSTREGEXP ")$"

#define NIR TRUE

/*----------------------------------------------------------------------------
			     Functions prototypes
 ----------------------------------------------------------------------------*/

CPL_RECIPE_DEFINE(isaac_img_lingain, ISAAC_BINARY_VERSION,
                  irplib_detmon_lg_fill_parlist_nir_default
                  (recipe->parameters, RECIPE_NAME, PACKAGE_TARNAME),
                  "Lars Lundin", PACKAGE_BUGREPORT, "2008", 
                  "Linearity/Gain recipe for the IR domain",
                  irplib_detmon_lg_get_description(RECIPE_NAME, "ISAAC",
                                                   ISAAC_IMG_LINGAIN_ON_RAW,
                                                   ISAAC_IMG_LINGAIN_OFF_RAW));

/*---------------------------------------------------------------------------*/
/*
  @brief    Interpret the command line options and execute the data processing
  @param    frameset    the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/

static int isaac_img_lingain(cpl_frameset            * frameset,
			 const cpl_parameterlist * parlist)
{
    cpl_propertylist * lintbl = 
	irplib_detmon_fill_prolist("DET_LIN_INFO", "TYPE", "TECH", CPL_TRUE);

    cpl_propertylist * gaintbl = 
	irplib_detmon_fill_prolist("GAIN_INFO",    "TYPE", "TECH", CPL_TRUE);

    cpl_propertylist * coeffscube = 
	irplib_detmon_fill_prolist("COEFFS_CUBE",  "TYPE", "TECH", CPL_TRUE);

    cpl_propertylist * bpm = 
	irplib_detmon_fill_prolist("BP_MAP_NL",    "TYPE", "TECH", CPL_TRUE);

    cpl_propertylist * corr = 
	irplib_detmon_fill_prolist("AUTOCORR",     "TYPE", "TECH", CPL_TRUE);

    cpl_propertylist * diff_flat = 
	irplib_detmon_fill_prolist("DIFF_FLAT",    "TYPE", "TECH", CPL_TRUE);

    const cpl_error_code error = irplib_detmon_lg(frameset,
                                                  parlist,
                                                  ISAAC_IMG_LINGAIN_ON_RAW,
                                                  ISAAC_IMG_LINGAIN_OFF_RAW,
                                                  RECIPE_NAME,
	                                          PACKAGE_TARNAME,
                                                  PAFREGEXP,
                                                  lintbl, gaintbl, coeffscube,
						  bpm, corr, diff_flat,
                                                  PACKAGE "/" PACKAGE_VERSION,
                                                  NULL, NULL, NIR);

    cpl_propertylist_delete(lintbl);
    cpl_propertylist_delete(gaintbl);
    cpl_propertylist_delete(coeffscube);
    cpl_propertylist_delete(bpm);
    cpl_propertylist_delete(corr);
    cpl_propertylist_delete(diff_flat);

    /* Propagate the error, if any */
    cpl_ensure_code(!error, error);

    return CPL_ERROR_NONE;
}
